=== modified file 'drizzled/cursor.cc'
--- drizzled/cursor.cc	2009-11-16 17:06:48 +0000
+++ drizzled/cursor.cc	2009-11-16 20:50:08 +0000
@@ -112,9 +112,9 @@
   return inited == INDEX ? ha_index_end() : inited == RND ? ha_rnd_end() : 0;
 }
 
-Cursor::Table_flags Cursor::ha_table_flags() const
+plugin::StorageEngine::Table_flags Cursor::ha_table_flags() const
 {
-  return cached_table_flags;
+  return engine->table_flags();
 }
 
 void Cursor::ha_start_bulk_insert(ha_rows rows)
@@ -218,7 +218,6 @@
     }
     else
       dup_ref=ref+ALIGN_SIZE(ref_length);
-    cached_table_flags= table_flags();
   }
   return error;
 }
@@ -1460,8 +1459,6 @@
 
   int error= external_lock(session, lock_type);
 
-  if (error == 0)
-    cached_table_flags= table_flags();
   return error;
 }
 

=== modified file 'drizzled/cursor.h'
--- drizzled/cursor.h	2009-11-16 18:23:07 +0000
+++ drizzled/cursor.h	2009-11-16 20:50:08 +0000
@@ -163,13 +163,9 @@
 
 class Cursor :public Sql_alloc
 {
-public:
-  typedef uint64_t Table_flags;
-
 protected:
   TableShare *table_share;   /* The table definition */
   Table *table;               /* The current open table */
-  Table_flags cached_table_flags;       /* Set on init() and open() */
 
   ha_rows estimation_rows_to_insert;
 public:
@@ -195,6 +191,12 @@
     being scanned.
   */
   bool in_range_check_pushed_down;
+  bool is_ordered;
+  bool isOrdered(void)
+  {
+    return is_ordered;
+  }
+
 
   /** Current range (the one we're now returning rows from) */
   KEY_MULTI_RANGE mrr_cur_range;
@@ -240,6 +242,7 @@
     :table_share(&share_arg), table(0),
     estimation_rows_to_insert(0), engine(&engine_arg),
     ref(0), in_range_check_pushed_down(false),
+    is_ordered(false),
     key_used_on_scan(MAX_KEY), active_index(MAX_KEY),
     ref_length(sizeof(my_off_t)),
     inited(NONE),
@@ -248,11 +251,6 @@
     {}
   virtual ~Cursor(void);
   virtual Cursor *clone(MEM_ROOT *mem_root);
-  /** This is called after create to allow us to set up cached variables */
-  void init()
-  {
-    cached_table_flags= table_flags();
-  }
 
   /* ha_ methods: pubilc wrappers for private virtual API */
 
@@ -265,7 +263,7 @@
 
   /* this is necessary in many places, e.g. in HANDLER command */
   int ha_index_or_rnd_end();
-  Table_flags ha_table_flags() const;
+  drizzled::plugin::StorageEngine::Table_flags ha_table_flags() const;
 
   /**
     These functions represent the public interface to *users* of the
@@ -697,7 +695,6 @@
     by that statement.
   */
   virtual int reset() { return 0; }
-  virtual Table_flags table_flags(void) const= 0;
 
   /**
     Is not invoked for non-transactional temporary tables.

=== modified file 'drizzled/definitions.h'
--- drizzled/definitions.h	2009-11-04 16:17:49 +0000
+++ drizzled/definitions.h	2009-11-16 20:30:19 +0000
@@ -340,15 +340,6 @@
 #define HA_NO_TRANSACTIONS     (1 << 0) /* Doesn't support transactions */
 #define HA_PARTIAL_COLUMN_READ (1 << 1) /* read may not return all columns */
 #define HA_TABLE_SCAN_ON_INDEX (1 << 2) /* No separate data/index file */
-/*
-  The following should be set if the following is not true when scanning
-  a table with rnd_next()
-  - We will see all rows (including deleted ones)
-  - Row positions are 'table->s->db_record_offset' apart
-  If this flag is not set, filesort will do a postion() call for each matched
-  row to be able to find the row later.
-*/
-#define HA_REC_NOT_IN_SEQ      (1 << 3)
 
 /*
   Reading keys in random order is as fast as reading keys in sort order

=== modified file 'drizzled/enum.h'
--- drizzled/enum.h	2009-11-14 22:50:02 +0000
+++ drizzled/enum.h	2009-11-16 20:30:19 +0000
@@ -347,5 +347,4 @@
   DUP_UPDATE 
 };
 
-
 #endif /* DRIZZLED_ENUM_H */

=== modified file 'drizzled/filesort.cc'
--- drizzled/filesort.cc	2009-11-16 17:06:48 +0000
+++ drizzled/filesort.cc	2009-11-16 20:30:19 +0000
@@ -465,7 +465,7 @@
   ref_pos= ref_buff;
   quick_select=select && select->quick;
   record=0;
-  flag= ((!indexfile && file->ha_table_flags() & HA_REC_NOT_IN_SEQ)
+  flag= ((!indexfile && ! file->isOrdered())
 	 || quick_select);
   if (indexfile || flag)
     ref_pos= &file->ref[0];

=== modified file 'drizzled/plugin/storage_engine.cc'
--- drizzled/plugin/storage_engine.cc	2009-11-16 17:46:51 +0000
+++ drizzled/plugin/storage_engine.cc	2009-11-16 20:50:08 +0000
@@ -777,13 +777,8 @@
 
 Cursor *plugin::StorageEngine::getCursor(TableShare &share, MEM_ROOT *alloc)
 {
-  Cursor *file;
-
   assert(enabled);
-
-  if ((file= create(share, alloc)))
-    file->init();
-  return file;
+  return create(share, alloc);
 }
 
 /**

=== modified file 'drizzled/plugin/storage_engine.h'
--- drizzled/plugin/storage_engine.h	2009-11-16 17:06:48 +0000
+++ drizzled/plugin/storage_engine.h	2009-11-16 20:30:19 +0000
@@ -94,6 +94,11 @@
 */
 class StorageEngine : public Plugin
 {
+public:
+  typedef uint64_t Table_flags;
+
+private:
+
   /*
     Name used for storage engine.
   */
@@ -189,6 +194,8 @@
   */
   uint32_t slot;
 
+  virtual Table_flags table_flags(void) const= 0;
+
   inline uint32_t getSlot (void) { return slot; }
   inline void setSlot (uint32_t value) { slot= value; }
 

=== modified file 'plugin/archive/ha_archive.cc'
--- plugin/archive/ha_archive.cc	2009-11-13 18:55:43 +0000
+++ plugin/archive/ha_archive.cc	2009-11-16 20:30:19 +0000
@@ -145,6 +145,13 @@
     table_definition_ext= ARZ;
   }
 
+  uint64_t table_flags() const
+  {
+    return (HA_NO_TRANSACTIONS |
+            HA_STATS_RECORDS_IS_EXACT |
+            HA_HAS_RECORDS);
+  }
+
   virtual Cursor *create(TableShare &table,
                           MEM_ROOT *mem_root)
   {
@@ -1042,7 +1049,7 @@
 
 
 /*
-  Thanks to the table flag HA_REC_NOT_IN_SEQ this will be called after
+  Thanks to the table bool is_ordered this will be called after
   each call to ha_archive::rnd_next() if an ordering of the rows is
   needed.
 */

=== modified file 'plugin/archive/ha_archive.h'
--- plugin/archive/ha_archive.h	2009-11-11 19:42:27 +0000
+++ plugin/archive/ha_archive.h	2009-11-16 20:30:19 +0000
@@ -96,12 +96,6 @@
 
   const char *index_type(uint32_t)
   { return "NONE"; }
-  uint64_t table_flags() const
-  {
-    return (HA_NO_TRANSACTIONS | HA_REC_NOT_IN_SEQ |
-            HA_STATS_RECORDS_IS_EXACT |
-            HA_HAS_RECORDS);
-  }
   uint32_t index_flags(uint32_t, uint32_t, bool) const
   {
     return HA_ONLY_WHOLE_INDEX;

=== modified file 'plugin/blackhole/ha_blackhole.cc'
--- plugin/blackhole/ha_blackhole.cc	2009-11-13 18:55:43 +0000
+++ plugin/blackhole/ha_blackhole.cc	2009-11-16 20:30:19 +0000
@@ -46,6 +46,11 @@
     table_definition_ext= BLACKHOLE_EXT;
   }
 
+  uint64_t table_flags() const
+  {
+    return(HA_NULL_IN_KEY | HA_CAN_INDEX_BLOBS | HA_AUTO_PART_KEY);
+  }
+
   virtual Cursor *create(TableShare &table,
                           MEM_ROOT *mem_root)
   {

=== modified file 'plugin/blackhole/ha_blackhole.h'
--- plugin/blackhole/ha_blackhole.h	2009-11-11 19:42:27 +0000
+++ plugin/blackhole/ha_blackhole.h	2009-11-16 20:30:19 +0000
@@ -53,10 +53,6 @@
     don't implement this method unless you really have indexes
   */
   const char *index_type(uint32_t key_number);
-  uint64_t table_flags() const
-  {
-    return(HA_NULL_IN_KEY | HA_CAN_INDEX_BLOBS | HA_AUTO_PART_KEY);
-  }
   uint32_t index_flags(uint32_t inx, uint32_t part, bool all_parts) const;
   /* The following defines can be increased if necessary */
   uint32_t max_supported_keys()          const { return BLACKHOLE_MAX_KEY; }

=== modified file 'plugin/csv/ha_tina.cc'
--- plugin/csv/ha_tina.cc	2009-11-13 18:55:43 +0000
+++ plugin/csv/ha_tina.cc	2009-11-16 20:30:19 +0000
@@ -129,6 +129,11 @@
     return new (mem_root) ha_tina(*this, table);
   }
 
+  uint64_t table_flags() const
+  {
+    return (HA_NO_TRANSACTIONS | HA_NO_AUTO_INCREMENT);
+  }
+
   const char **bas_ext() const {
     return ha_tina_exts;
   }

=== modified file 'plugin/csv/ha_tina.h'
--- plugin/csv/ha_tina.h	2009-11-11 19:42:27 +0000
+++ plugin/csv/ha_tina.h	2009-11-16 20:30:19 +0000
@@ -101,10 +101,6 @@
   const char *table_type(void) const { return "CSV"; }
   const char *index_type(uint32_t)
   { return "NONE"; }
-  uint64_t table_flags() const
-  {
-    return (HA_NO_TRANSACTIONS | HA_REC_NOT_IN_SEQ | HA_NO_AUTO_INCREMENT);
-  }
   uint32_t index_flags(uint32_t, uint32_t, bool) const
   {
     /*

=== modified file 'plugin/heap/ha_heap.cc'
--- plugin/heap/ha_heap.cc	2009-11-13 18:55:43 +0000
+++ plugin/heap/ha_heap.cc	2009-11-16 20:30:19 +0000
@@ -46,6 +46,15 @@
     addAlias("HEAP");
   }
 
+  uint64_t table_flags() const
+  {
+    return (HA_FAST_KEY_READ |
+            HA_NO_BLOBS | HA_NULL_IN_KEY |
+            HA_NO_TRANSACTIONS |
+            HA_HAS_RECORDS |
+            HA_STATS_RECORDS_IS_EXACT);
+  }
+
   virtual Cursor *create(TableShare &table,
                           MEM_ROOT *mem_root)
   {

=== modified file 'plugin/heap/ha_heap.h'
--- plugin/heap/ha_heap.h	2009-11-11 19:42:27 +0000
+++ plugin/heap/ha_heap.h	2009-11-16 20:30:19 +0000
@@ -42,12 +42,6 @@
 
   const char *index_type(uint32_t inx);
   enum row_type get_row_type() const;
-  uint64_t table_flags() const
-  {
-    return (HA_FAST_KEY_READ | HA_NO_BLOBS | HA_NULL_IN_KEY |
-            HA_REC_NOT_IN_SEQ | HA_NO_TRANSACTIONS |
-            HA_HAS_RECORDS | HA_STATS_RECORDS_IS_EXACT);
-  }
   uint32_t index_flags(uint32_t inx, uint32_t part, bool all_parts) const;
   const key_map *keys_to_use_for_scanning() { return &btree_keys; }
   uint32_t max_supported_keys()          const { return MAX_KEY; }

=== modified file 'plugin/information_engine/information_cursor.h'
--- plugin/information_engine/information_cursor.h	2009-11-12 03:09:31 +0000
+++ plugin/information_engine/information_cursor.h	2009-11-16 20:30:19 +0000
@@ -35,14 +35,6 @@
   InformationCursor(drizzled::plugin::StorageEngine &engine, TableShare &table_arg);
   ~InformationCursor() {}
 
-  /*
-    The name of the index type that will be used for display
-    don't implement this method unless you really have indexes
-  */
-  uint64_t table_flags() const
-  {
-    return 0;
-  }
   uint32_t index_flags(uint32_t inx, uint32_t part, bool all_parts) const;
   /* The following defines can be increased if necessary */
   int open(const char *name, int mode, uint32_t test_if_locked);

=== modified file 'plugin/information_engine/information_engine.h'
--- plugin/information_engine/information_engine.h	2009-11-12 16:49:34 +0000
+++ plugin/information_engine/information_engine.h	2009-11-16 20:30:19 +0000
@@ -44,6 +44,11 @@
     pthread_mutex_destroy(&mutex);
   }
 
+  uint64_t table_flags() const
+  {
+    return 0;
+  }
+
   class Share 
   {
   private:

=== modified file 'plugin/innobase/handler/ha_innodb.cc'
--- plugin/innobase/handler/ha_innodb.cc	2009-11-16 17:46:51 +0000
+++ plugin/innobase/handler/ha_innodb.cc	2009-11-16 20:30:19 +0000
@@ -250,6 +250,17 @@
     addAlias("INNOBASE");
   }
 
+  uint64_t table_flags() const
+  {
+    return (HA_NULL_IN_KEY |
+            HA_CAN_INDEX_BLOBS |
+            HA_PRIMARY_KEY_REQUIRED_FOR_POSITION |
+            HA_PRIMARY_KEY_IN_READ_INDEX |
+            HA_PARTIAL_COLUMN_READ |
+            HA_TABLE_SCAN_ON_INDEX | 
+            HA_MRR_CANT_SORT);
+  }
+
   virtual
   int
   close_connection(
@@ -1355,14 +1366,6 @@
 ha_innobase::ha_innobase(drizzled::plugin::StorageEngine &engine_arg,
                          TableShare &table_arg)
   :Cursor(engine_arg, table_arg),
-  int_table_flags(HA_REC_NOT_IN_SEQ |
-		  HA_NULL_IN_KEY |
-		  HA_CAN_INDEX_BLOBS |
-		  HA_PRIMARY_KEY_REQUIRED_FOR_POSITION |
-		  HA_PRIMARY_KEY_IN_READ_INDEX |
-		  HA_PARTIAL_COLUMN_READ |
-		  HA_TABLE_SCAN_ON_INDEX | 
-                  HA_MRR_CANT_SORT),
   primary_key(0), /* needs initialization because index_flags() may be called 
                      before this is set to the real value. It's ok to have any 
                      value here because it doesn't matter if we return the
@@ -2497,18 +2500,6 @@
 }
 
 
-
-/****************************************************************//**
-Get the table flags to use for the statement.
-@return	table flags */
-UNIV_INTERN
-Cursor::Table_flags
-ha_innobase::table_flags() const
-/*============================*/
-{
-        return int_table_flags;
-}
-
 /****************************************************************//**
 Returns the index type. */
 UNIV_INTERN

=== modified file 'plugin/innobase/handler/ha_innodb.h'
--- plugin/innobase/handler/ha_innodb.h	2009-11-16 17:46:51 +0000
+++ plugin/innobase/handler/ha_innodb.h	2009-11-16 20:30:19 +0000
@@ -71,7 +71,6 @@
 	ulong		upd_and_key_val_buff_len;
 					/* the length of each of the previous
 					two buffers */
-	Table_flags	int_table_flags;
 	uint		primary_key;
 	ulong		start_of_scan;	/*!< this is set to 1 when we are
 					starting a table scan but have not
@@ -109,7 +108,6 @@
 	UNIV_INTERN enum row_type get_row_type() const;
 
 	UNIV_INTERN const char* index_type(uint key_number);
-	UNIV_INTERN Table_flags table_flags() const;
 	UNIV_INTERN uint32_t index_flags(uint idx, uint part, bool all_parts) const;
 	UNIV_INTERN uint32_t max_supported_keys() const;
 	UNIV_INTERN uint32_t max_supported_key_length() const;

=== modified file 'plugin/myisam/ha_myisam.cc'
--- plugin/myisam/ha_myisam.cc	2009-11-13 18:55:43 +0000
+++ plugin/myisam/ha_myisam.cc	2009-11-16 20:30:19 +0000
@@ -68,6 +68,19 @@
   ~MyisamEngine()
   { }
 
+  uint64_t table_flags() const
+  {
+    return (HA_NULL_IN_KEY |
+            HA_DUPLICATE_POS |
+            HA_CAN_INDEX_BLOBS |
+            HA_AUTO_PART_KEY |
+            HA_NO_TRANSACTIONS |
+            HA_HAS_RECORDS |
+            HA_STATS_RECORDS_IS_EXACT |
+            HA_NEED_READ_RANGE_BUFFER |
+            HA_MRR_CANT_SORT);
+  }
+
   virtual Cursor *create(TableShare &table,
                           MEM_ROOT *mem_root)
   {
@@ -502,17 +515,10 @@
                      TableShare &table_arg)
   : Cursor(engine_arg, table_arg),
     file(0),
-    int_table_flags(HA_NULL_IN_KEY |
-                    HA_DUPLICATE_POS |
-                    HA_CAN_INDEX_BLOBS |
-                    HA_AUTO_PART_KEY |
-                    HA_NO_TRANSACTIONS |
-                    HA_HAS_RECORDS |
-                    HA_STATS_RECORDS_IS_EXACT |
-                    HA_NEED_READ_RANGE_BUFFER |
-                    HA_MRR_CANT_SORT),
      can_enable_indexes(1)
-{}
+{
+  is_ordered= true;
+}
 
 Cursor *ha_myisam::clone(MEM_ROOT *mem_root)
 {
@@ -575,7 +581,8 @@
   if (!(test_if_locked & HA_OPEN_WAIT_IF_LOCKED))
     mi_extra(file, HA_EXTRA_WAIT_LOCK, 0);
   if (!table->s->db_record_offset)
-    int_table_flags|=HA_REC_NOT_IN_SEQ;
+    is_ordered= false;
+
 
   keys_with_parts.reset();
   for (i= 0; i < table->s->keys; i++)

=== modified file 'plugin/myisam/ha_myisam.h'
--- plugin/myisam/ha_myisam.h	2009-11-11 19:42:27 +0000
+++ plugin/myisam/ha_myisam.h	2009-11-16 20:30:19 +0000
@@ -35,7 +35,6 @@
 class ha_myisam: public Cursor
 {
   MI_INFO *file;
-  uint64_t int_table_flags;
   char    *data_file_name, *index_file_name;
   bool can_enable_indexes;
   int repair(Session *session, MI_CHECK &param, bool optimize);
@@ -45,7 +44,6 @@
   ~ha_myisam() {}
   Cursor *clone(MEM_ROOT *mem_root);
   const char *index_type(uint32_t key_number);
-  uint64_t table_flags() const { return int_table_flags; }
   int index_init(uint32_t idx, bool sorted);
   int index_end();
   uint32_t index_flags(uint32_t inx, uint32_t part, bool all_parts) const;

